% This program creates Figure 3 and the nine panels in Figure 4.

clear
close all

k_bar=1;    % Defines required reserves - it is defined as an increment over the 'zero' level for balances

load data_2011_goodbanks
%load Poole_2011

% Calculates Q
par_med_MoG=[0.00046:0.005:2.2925]; % Poole
mu_dist=GM_mix2.mu;
mu1=par_med_MoG*mu_dist(1);
mu2=par_med_MoG*mu_dist(2);

for i=1:length(par_med_MoG)
Q(i)=GM_mix2.PComponents(1)*mu1(i)+GM_mix2.PComponents(2)*mu2(i);
end

%% Figure 3
interest_required_reserves_fed=0.0025/360;  % Interest on required reserve balances (annual rate)
interest_excess_balances_fed=0.0025/360;    % Interest rates on excess balances (annual rate)
discount_window_rate_fed=0.0075/360;

rate_FF_Poole1=zeros(length(interest_required_reserves_fed),length(par_med_MoG));
for i=1:length(interest_required_reserves_fed)
    for j=1:length(par_med_MoG)
        [i,j]
        rate_FF_Poole1(i,j)=simulation_2011(interest_required_reserves_fed(i),interest_excess_balances_fed(i),k_bar,discount_window_rate_fed,par_med_MoG(j),1);
    end
end

lower_bound=interest_excess_balances_fed*360*100;   % Defines floor on rates
upper_bound=discount_window_rate_fed*360*100;       % Defines ceilling on rates
% rate_FF_Poole1=Rate_FF_Poole1;

figure(1)
plot(Q/k_bar,100*rate_FF_Poole1,'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound lower_bound],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound upper_bound],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,0.25,'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,0.75,'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.01])
print(['Poole1'],'-depsc2')

%% Figure 4
%% Figures 4 (a-c)
interest_required_reserves_fed=[0.005:0.0025:0.01]/360; % Interest on required reserve balances (annual rate)
interest_excess_balances_fed=[0.005:0.0025:0.01]/360;   % Interest rates on excess balances (annual rate)
discount_window_rate_fed=interest_excess_balances_fed+0.005/360;

rate_FF_Poole2A=zeros(length(interest_required_reserves_fed),length(par_med_MoG));
for i=1:length(interest_required_reserves_fed)
    for j=1:length(par_med_MoG)
        [i,j]
        rate_FF_Poole2A(i,j)=simulation_2011(interest_required_reserves_fed(i),interest_excess_balances_fed(i),k_bar,discount_window_rate_fed(i),par_med_MoG(j),1);
    end
end

lower_bound=interest_excess_balances_fed*360*100;   % Defines floor on rates
upper_bound=discount_window_rate_fed*360*100;       % Defines ceilling on rates
% rate_FF_Poole2A=Rate_FF_Poole2A;

figure(2)
plot(Q/k_bar,100*rate_FF_Poole2A(1,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(1) lower_bound(1)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(1)  upper_bound(1)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(1),'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound(1),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_1'],'-depsc2')

figure(3)
plot(Q/k_bar,100*rate_FF_Poole2A(2,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(2) lower_bound(2)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(2)  upper_bound(2)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(2),'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound(2),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_2'],'-depsc2')

figure(4)
plot(Q/k_bar,100*rate_FF_Poole2A(3,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(3) lower_bound(3)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(3) upper_bound(3)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(3),'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound(3),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_3'],'-depsc2')

%% Figures 4 (d-f)
interest_required_reserves_fed=0.0025/360; % Interest on required reserve balances (annual rate)
interest_excess_balances_fed=0.0025/360; % Interest rates on excess balances (annual rate)
discount_window_rate_fed=0.0075/360+[0.0025:0.0025:0.0075]/360;

rate_FF_Poole2B=zeros(length(discount_window_rate_fed),length(par_med_MoG));
for i=1:length(discount_window_rate_fed)
    for j=1:length(par_med_MoG)
        [i,j]
        rate_FF_Poole2B(i,j)=simulation_2011(interest_required_reserves_fed,interest_excess_balances_fed,k_bar,discount_window_rate_fed(i),par_med_MoG(j),1);
    end
end

lower_bound=interest_excess_balances_fed*360*100;   % Defines floor on rates
upper_bound=discount_window_rate_fed*360*100;       % Defines ceilling on rates
% rate_FF_Poole2B=Rate_FF_Poole2B;

figure(5)
plot(Q/k_bar,100*rate_FF_Poole2B(1,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound lower_bound],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(1)  upper_bound(1)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound,'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound(1),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_4'],'-depsc2')

figure(6)
plot(Q/k_bar,100*rate_FF_Poole2B(2,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound lower_bound],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(2)  upper_bound(2)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound,'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+.5,upper_bound(2),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_5'],'-depsc2')

figure(7)
plot(Q/k_bar,100*rate_FF_Poole2B(3,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound lower_bound],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound(3) upper_bound(3)],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound,'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound(3),'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_6'],'-depsc2')

%% Figures 4 (g-i)
interest_required_reserves_fed=[0 0.005 0.007]/360; % Interest on required reserve balances (annual rate)
interest_excess_balances_fed=[0 0.005 0.007]/360; % Interest on required reserve balances (annual rate)
discount_window_rate_fed=0.0075/360;

rate_FF_Poole2C=zeros(length(interest_required_reserves_fed),length(par_med_MoG));
for i=1:length(interest_required_reserves_fed)
    for j=1:length(par_med_MoG)
        [i,j]
        rate_FF_Poole2C(i,j)=simulation_2011(interest_required_reserves_fed(i),interest_excess_balances_fed(i),k_bar,discount_window_rate_fed,par_med_MoG(j),1);
    end
end

lower_bound=interest_excess_balances_fed*360*100;   % Defines floor on rates
upper_bound=discount_window_rate_fed*360*100;       % Defines ceilling on rates

figure(8)
plot(Q/k_bar,100*rate_FF_Poole2C(1,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(1) lower_bound(1)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound  upper_bound],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(1),'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound,'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_7'],'-depsc2')

figure(9)
plot(Q/k_bar,100*rate_FF_Poole2C(2,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(2) lower_bound(2)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound  upper_bound],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(2),'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound,'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_8'],'-depsc2')

figure(10)
plot(Q/k_bar,100*rate_FF_Poole2C(3,:),'bo','MarkerFaceColor','b','MarkerSize',4)
hold on
plot([0 Q(end)/k_bar+1],[lower_bound(3) lower_bound(3)],'r--','linewidth', 3)   % Adds the lower bound for rates
plot([0 Q(end)/k_bar+1],[upper_bound  upper_bound],'r--','linewidth', 3) % Adds the upper bound for rates
hold off
grid
text(ceil(Q(end)/k_bar)+1.5,lower_bound(3)+0.15,'i_f','fontsize',15)
text(ceil(Q(end)/k_bar)+1.5,upper_bound-0.15,'i_f^w','fontsize',15)
test_string=['\textsf{$Q$/$\bar{k}$}'];
xlabel(test_string,'interpreter','latex','FontName','Helvetica','FontSize',15,'FontUnits','points');
title('Fed funds rate (%)','fontsize', 15)
axis([0 ceil(Q(end)/k_bar) 0 100*0.02])
print(['PooleA2_9'],'-depsc2')

% save Poole_2011

