% Goodness of fit, 2011

load data_2011_goodbanks.mat %%% Master file that contains master data: data_2011

%% ESTIMATION AND GOODNESS-OF-FIT
% Confidence level
alpha = 0.01;      % Confidence level

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% t location-scale distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Computing parameter estimates 
parameters_mle=mle(mean_data_2011_nooutliers,'distribution','tlocationscale');
mu_mean_data_2011=parameters_mle(1); 
sigma_mean_data_2011=parameters_mle(2);
nu_mean_data_2011=parameters_mle(3);
t_loc=fitdist(mean_data_2011_nooutliers,'tlocationscale');
t_loc.NLogL % Loglikelihood

%% 2. Goodness of fit
% If x has a t location-scale distribution, with parameters , sigma,and nu, then 
% y=(x- )/sigma has a Student's t distribution with nu degrees of freedom. To perform 
% the KS test we normalize the distribution and test if y_hat are from a
% Student's t distribution with nu_mean_data_2011_nooutliers degrees
% of freedom.

mean_data_2011_ajusted=(mean_data_2011_nooutliers-mu_mean_data_2011)./sigma_mean_data_2011;

% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(3);
PD=ProbDistUnivParam('tlocationscale',[mu_mean_data_2011;sigma_mean_data_2011;nu_mean_data_2011]);
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (t location-scale)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_tlocscale_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_ajusted,[mean_data_2011_ajusted tcdf(mean_data_2011_ajusted,nu_mean_data_2011)],alpha)

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_ajusted,'cdf',{@tcdf,nu_mean_data_2011},'nparams',1,'alpha',alpha)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Poisson
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Poisson distribution only takes nonnegative integer values. We adjust the data. 
% mean_data_2011_pos(:,1)=mean_data_2011_nooutliers-1.1*min(mean_data_2011_nooutliers);
% mean_data_2011_pos_round_nooutliers=round(mean_data_2011_pos(:,1));

% Computing parameter estimates 
parameters_mle=mle(mean_data_2011_pos_round_nooutliers,'distribution','poisson');
lambda_a=parameters_mle;
poiss=fitdist(mean_data_2011_pos_round_nooutliers,'poisson');
poiss.NLogL % Loglikelihood

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(4);
PD=ProbDistUnivParam('poisson',lambda_a);
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Poisson)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_poisson_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_pos_round_nooutliers,[mean_data_2011_pos_round_nooutliers poisscdf(mean_data_2011_pos_round_nooutliers,lambda_a)],alpha)

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_pos_round_nooutliers,'cdf',{@poisscdf,lambda_a},'nparams',1,'alpha',alpha)%,'edges',[1.0000 5 18 187.0000])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Negative binomial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % 1. Estimation
% % Negative binomial distribution only takes nonnegative integer values. We adjust the data. 
% mean_data_2011_pos(:,1)=mean_data_2011_nooutliers-1.1*min(mean_data_2011_nooutliers);
% mean_data_2011_pos_round_nooutliers=round(mean_data_2011_pos(:,1));
% Computing parameter estimates 
% The negative binomial has two paramenters: r, number of successes and p, 
% probability of success in a single trial
parameters_mle=mle(mean_data_2011_pos_round_nooutliers,'distribution','nbin');
r_a=parameters_mle(1);
p_a=parameters_mle(2);
neg_bin=fitdist(mean_data_2011_pos_round_nooutliers,'nbin');
neg_bin.NLogL

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(5);
PD=ProbDistUnivParam('nbin',[r_a;p_a]);
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Negative binomial)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_negbinomial_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_pos_round_nooutliers,[mean_data_2011_pos_round_nooutliers nbincdf(mean_data_2011_pos_round_nooutliers,r_a,p_a)],alpha)

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_pos_round_nooutliers,'cdf',{@nbincdf,r_a,p_a},'nparams',2,'alpha',alpha)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Normal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Computing parameter estimates 
parameters_mle=mle(mean_data_2011_nooutliers,'distribution','normal');
MUHAT=parameters_mle(1);
SIGMAHAT=parameters_mle(2);
nor=fitdist(mean_data_2011_nooutliers,'normal');
nor.NLogL

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(6);
PD=ProbDistUnivParam('normal',[MUHAT;SIGMAHAT]);
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Normal)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_normal_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_nooutliers,[mean_data_2011_nooutliers normcdf(mean_data_2011_nooutliers,MUHAT,SIGMAHAT)],alpha)

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_nooutliers,'cdf',{@normcdf,MUHAT,SIGMAHAT},'nparams',2,'alpha',alpha)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Logistic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Computing parameter estimates
parameters_mle=mle(mean_data_2011_nooutliers,'distribution','logistic');
loc=parameters_mle(1);
scal=parameters_mle(2);
logis=fitdist(mean_data_2011_nooutliers,'logistic');
logis.NLogL

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(7);
PD=ProbDistUnivParam('logistic',[loc;scal]);
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Logistic)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_logistic_2011'],'-depsc2')


% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_nooutliers,[mean_data_2011_nooutliers logisticcdf(mean_data_2011_nooutliers,loc,scal)],alpha)

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_nooutliers,'cdf',@(t)logisticcdf(t,loc,scal),'nparams',2,'alpha',alpha,'edges',[-68.3720 -8.8376 0 11.0072 130.0760])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generalized extreme value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Computing parameter estimates
parameters_mle=mle(mean_data_2011_nooutliers,'distribution','gev');
K_HAT=parameters_mle(1);
SIGMA_HAT=parameters_mle(2);
MU_HAT=parameters_mle(3);
geval=fitdist(mean_data_2011_nooutliers,'gev');
geval.NLogL

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
HH=figure(8);
PD=ProbDistUnivParam('gev',[K_HAT;SIGMA_HAT;MU_HAT])
qqplot(mean_data_2011_nooutliers,PD)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Generalized extreme value)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_Genextremevalue_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test  
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_nooutliers,[mean_data_2011_nooutliers gevcdf(mean_data_2011_nooutliers,K_HAT,SIGMA_HAT,MU_HAT)],alpha)

% (ii) Chi-square goodness-of-fit test
% [h,p,stats] = chi2gof(mean_data_2011_nooutliers,'cdf',{@gevcdf,K_HAT,SIGMA_HAT,MU_HAT},'nparams',3,'alpha',alpha,'edges',[-68.3720 -8.8376 0 11.0072 30.8520 130.0760])
[h,p,stats] = chi2gof(mean_data_2011_nooutliers,'cdf',{@gevcdf,K_HAT,SIGMA_HAT,MU_HAT},'nparams',3,'alpha',alpha)%,'edges',[-68.3720 -8.8376 0 11.0072 30.8520 130.0760])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Mixture of gaussians
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Estimation
% Computing parameter estimates
GM_mix2=gmdistribution.fit(mean_data_2011_nooutliers,2)
GM_mix2.NlogL % Negative Log-likelihood

%% 2. Goodness of fit
% 2a. Visual test for goodness of fit: Quantile-quantile plots
N=10000;
y=[sqrt(GM_mix2.Sigma(:,:,1))*randn(1,round(GM_mix2.PComponents(1)*N))+GM_mix2.mu(1) sqrt(GM_mix2.Sigma(:,:,2))*randn(1,round(GM_mix2.PComponents(2)*N))+GM_mix2.mu(2)];
HH=figure(9);
qqplot(mean_data_2011_nooutliers,y)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
xlabel('Fitted quantiles (Mixture of 2 gaussians)', 'fontsize', 15)
ylabel('Empirical quantiles', 'fontsize', 15)
title('2011')
print(['qqplot_2Gaussians_2011'],'-depsc2')

% 2b. Formal tests of goodness of fit
% (i) Kolmogorov-Smirnov Test
[H,pval,KSSTAT,CriticalValue]=kstest(mean_data_2011_nooutliers,[mean_data_2011_nooutliers cdf(GM_mix2,mean_data_2011_nooutliers)],alpha) % Kolmogorov-Smirnov 

% (ii) Chi-square goodness-of-fit test
[h,p,stats] = chi2gof(mean_data_2011_nooutliers,'cdf',@(z)cdf(GM_mix2,z'),'nparams',5,'alpha',alpha)%,'edges',[-68.3720 -2 -1 0 1 2 5 130.0760])

gm_mix2_cdf=cdf(GM_mix2,[min(mean_data_2011_nooutliers):max(mean_data_2011_nooutliers)]');
gm_mix2_pdf=pdf(GM_mix2,[min(mean_data_2011_nooutliers)-50:max(mean_data_2011_nooutliers)]');

figure(2)
histnorm(mean_data_2011_nooutliers,40)
hold on
plot([min(mean_data_2011_nooutliers)-50:max(mean_data_2011_nooutliers)]',gm_mix2_pdf,'r','linewidth', 3)
% H1=get(HH,'Children');
% set(H1(1),'fontsize', 20);
hold off
ylabel('Proportion of banks', 'fontsize', 15)
xlabel('4pm imputed balances/RoB', 'fontsize', 15)
title('2011')
grid
print -depsc2 gm_mix2_pdf_2011
