% Parameters for the simulations

% Simulations Variables:
% H=1000000; % Number of repetitions.
H=100000; % Number of repetitions.

rand('twister',1) % To fix the seed


% Parameter values
T=2.5/24;               % Length of the trading session (measured in days: 2.5 hours/24 (hours/day)) 
K=250;                  % # of different types of banks: K+1 (0,1,...,249)
K_neg=100;              
k_0=K_neg;              % Defines the `zero' for balances

delta=2.5/24;            % Length of repayment period (measured in days - loans repaid at t=T+delta)
delta_F=2.5/24;         % Length of fed funds market closing period (during the week the market is closed 2.5 hours) (measured in days)
delta_f_r=2.5/24;       % Length of period betweeen end of session and payment of IOR (or defficiency charges)
delta_f_o=2.5/24;       % Length of period betweeen end of session and payment of overnight overdafts penalties
delta_f_d=0;            % Length of period betweeen the time when intraday interest is earned (or owed) and the time when it is paid.
delta_f_c=2.5/24;       % Length of period betweeen end of session and payment of deficiency charges
delta_f_w=2.5/24;       % Length of period betweeen end of session and payment of discount window charges

r=0;           % Discount rate (daily rate)

% Policy rates
intraday_rate_fed=0.0000001/360;                                % Interest on intraday balances balances (annual rate)
daylight_overdraft_rate_fed=0.0036/360;                         % Daylight overdraft charge (annual rate)
overnight_overdraft_rate_fed=0.09249/360;                        % 400 basis points over the effective federal funds rate (annual rate)
deficiency_charge_rate_fed=0.01/360+discount_window_rate_fed;   % 100 basis points over the primary credit rate

% Model rates
intraday_rate=intraday_rate_fed;                                                % Model interest on intraday balances balances (annual rate)
interest_required_reserves=(log(1+interest_required_reserves_fed))/delta_F;     % Model interest on required reserve balances (annual rate)
interest_excess_balances=(log(1+interest_excess_balances_fed))/delta_F;         % Model interest on excess balances (annual rate)
overnight_overdraft_rate=(log(1+overnight_overdraft_rate_fed))/delta_F;         % Model overnight overdraft charge (annual rate)
daylight_overdraft_rate=daylight_overdraft_rate_fed;                            % Model daylight overdraft charge (annual rate)
discount_window_rate=(log(1+discount_window_rate_fed))/delta_F;                 % Model discount window rate (annual rate)
deficiency_charge_rate=(log(1+deficiency_charge_rate_fed))/delta_F;             % Model deficiency charge rate (annual rate)


Penalty_w=0.0475/360;         % Additional penalty for borrowing from the discount window
Penalty_o=0.0475/360;         % Additional penalty for unauthorized extension of credit (reputational damage...)
Penalty_c=0.0475/360;         % Additional penalty for end-of-day balances below required balances

epsilon=10^(-6);            % To add concavity to intraday pay-offs


%% Mixture of Gaussian with the data from 2007
if(flag_change_MoG==1) % To change means to: mu1=mu1_est*par_med_MoG and mu2=mu2_est*par_med_MoG so the ratio between mu1 and mu2 remains constant. We do not change the estimated variances.
    load data_2007_goodbanks
    m_ini=weights_mog(K-K_neg,K_neg,par_med_MoG*GM_mix2.mu,[sqrt(GM_mix2.Sigma(:,:,1));sqrt(GM_mix2.Sigma(:,:,2))],GM_mix2.PComponents);
    Q=[-K_neg:K-K_neg]*m_ini';                                         % Total assets
elseif(flag_change_MoG==2) % To change means to: mu1=mu1_est*par_med_MoG and mu2=mu2_est*par_med_MoG so the ratio between mu1 and mu2 remains constant. We change the estimated variances to make the whole variance constant.
    load data_2007_goodbanks
    sig=[sqrt(GM_mix2.Sigma(:,:,1));sqrt(GM_mix2.Sigma(:,:,2))];
    gam=sig(2)/sig(1);
    mu=GM_mix2.mu;
    lam=mu(2)/mu(1);
    sig2_o=GM_mix2.PComponents(1)*(sig(1)^2+mu(1)^2)+GM_mix2.PComponents(2)*(sig(2)^2+mu(2)^2)-(GM_mix2.PComponents(1)*mu(1)+GM_mix2.PComponents(2)*mu(2))^2;
    
    mu=par_med_MoG*mu;
    
    sig(1)=sqrt((sig2_o+((GM_mix2.PComponents(1)+lam*GM_mix2.PComponents(2))^2-(GM_mix2.PComponents(1)+lam^2*GM_mix2.PComponents(2)))*mu(1)^2)/(GM_mix2.PComponents(1)+gam^2*GM_mix2.PComponents(2)));
    sig(2)=gam*sig(1);
        
    m_ini=weights_mog(K-K_neg,K_neg,mu,sig,GM_mix2.PComponents);
    Q=[-K_neg:K-K_neg]*m_ini';                                         % Total assets
elseif(flag_change_MoG==3) % To change standard deviations to sig1=sig1_est*par_med_MoG and sig2=sig2_est*par_med_MoG so the ratio between sig1 and sig2 remains constant. We do not change the estimated means.
    load data_2007_goodbanks
    sig=[sqrt(GM_mix2.Sigma(:,:,1));sqrt(GM_mix2.Sigma(:,:,2))];
    sig=par_med_MoG*sig;
        
    m_ini=weights_mog(K-K_neg,K_neg,GM_mix2.mu,sig,GM_mix2.PComponents);
    Q=[-K_neg:K-K_neg]*m_ini';
elseif(flag_change_MoG==4) %%%% Careful with definition!
    load data_2007_goodbanks
    m_ini=weights_mog(K-K_neg,K_neg,par_med_MoG*[GM_mix2.mu(1) GM_mix2.mu(1)]',[sqrt(GM_mix2.Sigma(:,:,1));sqrt(GM_mix2.Sigma(:,:,2))/10],[0 1]);
    Q=[-K_neg:K-K_neg]*m_ini';      
elseif(flag_change_MoG==5) %%%% Careful with definition!
    load data_2007_goodbanks
    m_ini=weights_mog(K-K_neg,K_neg,par_med_MoG*GM_mix2.mu,[sqrt(GM_mix2.Sigma(:,:,1));sqrt(GM_mix2.Sigma(:,:,2))],[1 0]);
    Q=[-K_neg:K-K_neg]*m_ini';                                       % Total assets
end


% End-of-day payoffs
% Note:exp(overnight_overdraft_rate*delta_F)-1=overnight_overdraft_rate_fed
F=zeros(1,K+1);
for k=0:K
   if(k-k_0<0)          %k-k_0<0
       %F_o=exp(-r*delta_f_o)*(exp(overnight_overdraft_rate*delta_F)-1)*(k-k_0)+exp(-r*delta_f_o)*Penalty_o*(k-k_0)-exp(-r*delta_f_c)*(exp(deficiency_charge_rate*delta_F)-1)*(k_bar+k_0-k)-exp(-r*delta_f_c)*Penalty_c*(k_bar+k_0-k);
       % Old formulation
       F_o=exp(-r*delta_f_o)*(exp(overnight_overdraft_rate*delta_F)-1)*(k-k_0)+exp(-r*delta_f_o)*Penalty_o*(k-k_0)-exp(-r*delta_f_c)*(exp(deficiency_charge_rate*delta_F)-1)*(k_bar)-exp(-r*delta_f_c)*Penalty_c*k_bar;
       F_w_k_bar=exp(-r*delta_f_r)*(exp(interest_required_reserves*delta_F)-1)*k_bar-exp(-r*delta_f_w)*(exp(discount_window_rate*delta_F)-1)*(k_bar+k_0-k)-exp(-r*delta_f_w)*Penalty_w*(k_bar+k_0-k);         
       F_w_0=exp(-r*delta_f_w)*(exp(discount_window_rate*delta_F)-1)*(k-k_0)+exp(-r*delta_f_w)*Penalty_w*(k-k_0)-exp(-r*delta_f_c)*(exp(deficiency_charge_rate*delta_F)-1)*k_bar-exp(-r*delta_f_c)*Penalty_c*k_bar;
       F(k+1)=max([F_w_k_bar,F_w_0,F_o]); %If indifferent, banks goes to the discount window
   elseif(k-k_0<k_bar)  %0<=k-k_0<k_bar
       F_c=exp(-r*delta_f_r)*(exp(interest_required_reserves*delta_F)-1)*(k-k_0)-exp(-r*delta_f_c)*(exp(deficiency_charge_rate*delta_F)-1)*(k_bar+k_0-k)-exp(-r*delta_f_c)*Penalty_c*(k_bar+k_0-k);
       F_w_k_bar=exp(-r*delta_f_r)*(exp(interest_required_reserves*delta_F)-1)*k_bar-exp(-r*delta_f_w)*(exp(discount_window_rate*delta_F)-1)*(k_bar+k_0-k)-exp(-r*delta_f_w)*Penalty_w*(k_bar+k_0-k);     
       F(k+1)=max(F_w_k_bar,F_c); %If indifferent, banks goes to the discount window
   else                 %k_bar<=k-k_0           % F_e
       F(k+1)=exp(-r*delta_f_r)*(exp(interest_required_reserves*delta_F)-1)*k_bar+exp(-r*delta_f_r)*(exp(interest_excess_balances*delta_F)-1)*(k-k_0-k_bar);
   end
end

U=exp(-r*delta_F)*([0:K]-k_0)+F;


% Intraday payoffs
u=zeros(1,K+1);
for k=0:K
   if(k-k_0<0)          %k-k_0<0
       u(k+1)=-exp(-r*delta_f_d)*(k_0-k)^(1+epsilon)*daylight_overdraft_rate;
   else                 %k-k_0>=0
       u(k+1)=exp(-r*delta_f_d)*(k-k_0)^(1-epsilon)*intraday_rate;
   end
end
