function [rate_FF,Q]=simulation_2011(interest_required_reserves_fed,interest_excess_balances_fed,k_bar,discount_window_rate_fed,par_med_MoG,flag_change_MoG)

parameters_2011
solution_K_mod      % This file solves for the equilibrium (measures and value functions). It also loads a file with the value of the parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIMULATING THE PATHS
%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%% Number of meetings
% Generates random realizations from a Poisson distribution with parameter alfa*T
N=poissrnd(alfa*T,H,1);  % Array of (H,1) random numbers chosen from the Poisson distribution with parameter alfa*T -- Number of meeting times in one session. H=1000 repetitions (banks).


%% Time of the meetings
% Generates N random realizations from a uniform distribution on [0,T]
Times=-ones(H,max(N)); % Times is a matrix of size H x max(N), where H is the number of repetitions (can think of H as # of banks) and max(N) is the maximum number of meetings (a bank has) during the trading session
for i=1:H
    Times(i,1:N(i))=sort(T*rand(1,N(i))); % Matrix of (H,max(N)) (ordered) random numbers from a uniform distribution on [0,T] -- Given the number of meetings N for each bank (repetition), 'Times' gives the time when the meetings take place
end
%% Initial balances for the H=1000 banks
% Generate random realizations from a multinomial distribution
m_ini(find(m_ini<0))=0;
aux=mnrnd(1,m_ini,H); % aux returns H random vectors (of size K+1) chosen from the multinomial distribution with the same probabilities 
                      % given by m_ini (initial distribution of balances). Each row of aux corresponds to one multinomial
                      % outcome. The parameter N=1 indicates that the number of trials for each multinomial outcome is 1 (i.e. the sample size = 1). 
                      % It creates the initial balances of H=1000 banks
                      % according to the initial distribution of balances m_ini.
                      % H x K+1 matrix of 0s and 1s. Value equal to 1 indicates the balances of each bank
                      

M=-ones(H,max(N)+1); % M is a matrix of size H x {max(N)+1}, where H is the number of banks and max(N) is the maximum number of meetings a bank has
% Calculates initial balances for the H=1000 banks
for i=0:K
    M(find(aux(:,i+1)==1),1)=i; % M(:,1)is a H x 1 vector of the initial balances for all banks
                                % 'find' gives the position (identifying the banks) for which the condition in parenthesis is true.
                                % aux(:,i+1)==1 -- for a given column (say i=0) this condition is true for the banks with zero balances (for which k=0)
end


%% Post-trade balances and measures of lending (sales), borrowing (purchases) and intermediation
Purchases=zeros(H,1);
Sales=zeros(H,1);
X=zeros(H,1);
IntermediatedFunds=zeros(H,1);
Lenders=zeros(H,1);
Borrowers=zeros(H,1);
Counterparties=zeros(H,1);
everLenders=zeros(H,1);
everBorrowers=zeros(H,1);
everCounterparties=zeros(H,1);
M_end=zeros(H,1);

PURCHASES=zeros(size(Times));

R=-ones(H,max(N));
rho=-ones(H,max(N));
rho_FF=-ones(H,max(N));

% Post-trade balances and bank-level variables of interest
for i=1:H % For every path (bank)
    for j=1:N(i)% For every meeting time 
        m=approx_m(Times(i,j));     % m is the vector of balances of bank i at meeting time j. 
                                    % m is the probability distribution over the balances of potential trading partners
                                    % m gives the value of the measures calculated using an approximate solution instead of solving the ode in every iteration 
        m(find(m<0))=0;
        m=m/sum(m);
        
        M_enc=find(mnrnd(1,m,1))-1; % M_enc gives the type m (balance) of the trading partner  
                                    
                                    % mnrnd(1,m,1) returns 1 random vector (of size K+1) chosen from the multinomial distribution with the same probabilities 
                                    % given by m (distribution of balances of bank i at meeting time j).  
                                    
                                    % 'find' gives the position for which the condition in parenthesis is not equal to zero identifying the balance of the trading partner
                                    % The code substracts 1 so that it lies
                                    % between 0 and K since "find" finds components not equal to zero and returns the position where the '1' is (in this case a value between 1 and K+1).
        
       
        
        
        % Post-trade balances -- Sharing of balances between banks:                             
        if(mod(M(i,j)+M_enc,2)==0)          % if total holdings of reserves is an even number... 
            M(i,j+1)=(M(i,j)+M_enc)/2;      % ... midpoint is the equilibrium outcome
            PURCHASES(i,j)=M(i,j+1)-M(i,j); % Size of the loan
        else                                % if total holdings of reserves is an odd number...
%             if(rand>.5)                   % ... we randomize who gets one more unit 
%                 M(i,j+1)=(M(i,j)+M_enc+1)/2;
%             else
%                 M(i,j+1)=(M(i,j)+M_enc-1)/2;
%             end                           
            if(M(i,j)>M_enc)                    % if total holdings of reserves is an odd number...
                M(i,j+1)=(M(i,j)+M_enc+1)/2;    % ... the one with more units before trade gets one more unit 
                PURCHASES(i,j)=M(i,j+1)-M(i,j); % Size of the loan
            else
                M(i,j+1)=(M(i,j)+M_enc-1)/2;    % ... the one with more units before trade gets one more unit 
                PURCHASES(i,j)=M(i,j+1)-M(i,j); % Size of the loan
            end
        end
        
        % Calculates the value functions at the time of the meeting t=Times(i,j) 
        v_app=approx_v(T-Times(i,j),tau,v); % Works in tau and gives an approximated solution (linear approximation).        
        i_simu=M(i,j);                  % Balance of a bank entering the meeting with i balances
        j_simu=M_enc;                   % Balance of a bank entering the meeting with j balances
        k_simu=M(i,j+1);                % Balance of a bank leaving the meeting with k balances
        s_simu=i_simu+j_simu-k_simu;    % Balance of a bank leaving the meeting with s balances
        
        if(j_simu~=s_simu)  % If there is trade after the meeting (i.e. if the size of the loan is not zero)
            R(i,j)=exp(r*((T-Times(i,j))+delta))/2.*(v_app(k_simu+1)-v_app(i_simu+1)+v_app(j_simu+1)-v_app(s_simu+1)); % Repayment
            rho(i,j)=log(R(i,j)/(j_simu-s_simu))./((T-Times(i,j))+delta);   % Loan rate
            rho_FF(i,j)=exp(rho(i,j)*((T-Times(i,j))+delta))-1;
        end
        
    end
    
    % Bank-level measures: Variables of interest for each bank i
    Purchases(i)=sum(max(M(i,2:N(i)+1)-M(i,1:N(i)),0));             % Total purchases (as the difference between post-trade balancs and pre-trade balances). 
                                                                    % Purchases(i)=0 if bank i did not purchase any funds.
                                                                                                                                
    Sales(i)=sum(-min(M(i,2:N(i)+1)-M(i,1:N(i)),0));                % Total Sales (as the difference between post-trade balancs and pre-trade balances)
                                                                    % Sales(i)=0 if bank i did not purchase any funds.
                                                                    
    X(i)=Purchases(i)+Sales(i)-abs(Purchases(i)-Sales(i));          % Excess funds reallocation - Measure of intermediation: aggregate cumulative gross reallocation of funds
    if((Purchases(i)+Sales(i))==0) 
        IntermediatedFunds(i)=0;                                    % We define 0/0=0 
    else
        IntermediatedFunds(i)=X(i)/(Purchases(i)+Sales(i));         % Proportion of intermediated funds
    end
          
    Lenders(i)=length(find(max(M(i,2:N(i)+1)-M(i,1:N(i)),0)));      % Number of lenders bank i borrowed from
    Borrowers(i)=length(find((-min(M(i,2:N(i)+1)-M(i,1:N(i)),0)))); % Number of borrowers bank i lend to
    Counterparties(i)=Lenders(i)+Borrowers(i);                      % Number of counterparties bank i has (both borrowers and lenders)
    
    everLenders(i)=Lenders(i)>0;                                    % Indicates if bank i has at least a lender (i.e. has borrowed from at least a lender)
    everBorrowers(i)=Borrowers(i)>0;                                % Indicates if bank i has at least a borrower (i.e. has lent to at least a borrower)
    everCounterparties(i)=everLenders(i).*everBorrowers(i);         % Indicates if bank i has at least one counterparty (either a borrower or a lender)
    
    M_end(i,1)=M(i,N(i)+1);                                         % Final holdings - bank i's balance at the end of the trading session
    
%     % Variables to check what's going on
%     M(i,1:N(i)+1)                       % Balances
%     max(M(i,2:N(i)+1)-M(i,1:N(i)),0)    % Purchases
%     -min(M(i,2:N(i)+1)-M(i,1:N(i)),0)   % Sales
%     [Purchases(i) Sales(i) X(i) Lenders(i) Borrowers(i) everLenders(i) everBorrowers(i)]
%     pause
end

TRADES=[PURCHASES(PURCHASES>=1);(-PURCHASES(PURCHASES<=-1))]; % Loan sizes
% TRADES=abs(PURCHASES(PURCHASES~=0)); % Another way of doing it.


fine_slice_number=150;     % Defines a finer partition (seconds)
M_time=M(:,1)*ones(1,fine_slice_number+1);
for i=1:H
    aux_t=ceil(Times(i,1:N(i))/T*fine_slice_number);
    for j=1:N(i)
        M_time(i,aux_t(j)+1:end)=M(i,j+1);  % Balances each second
    end
end

% Purchases and sales over time
Slice_number=150;               % Slice_number=150 defines 1-minute intervals.  Slice_number=300 for 30-second intervals. 15 for each 10-minute intervals. 
TRADES_time=zeros(Slice_number,1);
L_TRADES_time=zeros(Slice_number,1);
S_PUR=zeros(Slice_number,1);  
N_PUR=zeros(Slice_number,1);
S_SAL=zeros(Slice_number,1);
N_SAL=zeros(Slice_number,1);
AS_PUR=zeros(Slice_number,1);  
AN_PUR=zeros(Slice_number,1);
AS_SAL=zeros(Slice_number,1);
AN_SAL=zeros(Slice_number,1);
H_m_PUR=zeros(Slice_number,floor(K/2));
H_m_SAL=zeros(Slice_number,floor(K/2));

RHO_market=zeros(Slice_number,1);
L_RHO_sliced=zeros(Slice_number,1);
RHO_market_FF=zeros(Slice_number,1);
L_RHO_sliced_FF=zeros(Slice_number,1);

PUR_b=-ones(ceil(H/10),Slice_number);
SAL_w=-ones(ceil(H/10),Slice_number);
L_PUR_b=zeros(1,Slice_number);
L_SAL_w=zeros(1,Slice_number);

for i=1:Slice_number % For each minute (or interval)
    index=find(Times(:)>T/Slice_number*(i-1) & Times(:)<=T/Slice_number*i);
    
    TRADES_time(i,1:length(index))=abs(PURCHASES(index));   % Loan sizes at each minute (purchases and sales)
    L_TRADES_time(i)=length(index);                          
    
    PUR=max(PURCHASES(index),0);    % Amount of all purchases (if 2 or more purchases occur in the same minute, they are registered separately).
    S_PUR(i)=sum(PUR);              % Total amount purchased in that minute
    N_PUR(i)=length(find(PUR));     % Number of purchases in that minute
    SAL=max(-PURCHASES(index),0);   % Amount of all sales (if 2 or more sales occur in the same minute, they are registered separately).
    S_SAL(i)=sum(SAL);              % Total amount sold in that minute
    N_SAL(i)=length(find(SAL));     % Number of Sales in that minute
    if(i==1)
        AS_PUR(1)=S_PUR(1);                 % In the first minute, the cumulative amount of funds purchased equals the amount in the first minute.
        AN_PUR(1)=N_PUR(1);                 % In the first minute, the cumulative number of purchases equals the number of purchases in the first minute.
        AS_SAL(1)=S_SAL(1);                 % In the first minute, the cumulative amount of funds sold equals the amount in the first minute.
        AN_SAL(1)=N_SAL(1);                 % In the first minute, the cumulative number of sales equals the number of sales in the first minute.
    else
        AS_PUR(i)=AS_PUR(i-1)+S_PUR(i);     % Cumulative amount of fed funds purchased (minute by minute)
        AN_PUR(i)=AN_PUR(i-1)+N_PUR(i);     % Cumulative number of fed funds purchases (minute by minute)
        AS_SAL(i)=AS_SAL(i-1)+S_SAL(i);     % Cumulative amount of fed funds (sold minute by minute)
        AN_SAL(i)=AN_SAL(i-1)+N_SAL(i);     % Cumulative number of fed funds sales (minute by minute)
    end 
    for j=1:floor(K/2)
        H_m_PUR(i,j)=length(find(PUR==j));  % Number of purchases of sizes 1,2,3,...,floor(K/2) each minute
        H_m_SAL(i,j)=length(find(SAL==j));  % Number of sales of sizes 1,2,3,...,floor(K/2) each minute
    end
    
    
    RHO=rho(index);                                         % Vector of the rates of all loans traded at that minute. Since rho=-ones(H,max(N)), the rate corresponding to a minute where a bank did not trade is set to -1 
    RHO_market(i)=sum((PUR+SAL).*RHO)/(S_PUR(i)+S_SAL(i));  % Calculates the market rate at that minute (value-weighted)
    RHO_sliced(i,1:length(RHO(RHO>-1)))=RHO(RHO>-1);        % Vector of the rates of all loans traded at that minute which is similar to RHO but where the -1 are eliminated. A (-1) ocurrs when there is no trade after a meeting or no meeting at time t.
    L_RHO_sliced(i)=length(RHO(RHO>-1));                    % Length of RHO_sliced
    
    RHO_FF=rho_FF(index);                                           % Vector of the (Fed) rates of all loans traded at that minute.
    RHO_market_FF(i)=sum((PUR+SAL).*RHO_FF)/(S_PUR(i)+S_SAL(i));    % Market (Fed) rate at that minute (value-weighted)
    RHO_sliced_FF(i,1:length(RHO_FF(RHO_FF>-1)))=RHO_FF(RHO_FF>-1); % Similar to RHO but eliminating -1 that ocurred when there is no trade after a meeting.
    L_RHO_sliced_FF(i)=length(RHO_FF(RHO_FF>-1));
       
    L_neg=length(find(M_time(:,1+(i-1)*fine_slice_number/Slice_number)-k_0-k_bar<0));  % Length of the vector of banks with negative adjusted balances (k-k_0-k_bar<0) at each second 
    L_pos=length(find(M_time(:,1+(i-1)*fine_slice_number/Slice_number)-k_0-k_bar>=0)); % Length of the vector of banks with nonnegative adjusted balances (k-k_0-k_bar>=0) at each second 
    
    [val,pos]=sort(M_time(:,1+(i-1)*fine_slice_number/Slice_number));       % Orders banks' balances each in increasing order. Returns both the value and the position
       
    I_worse_10percent=pos(1:round(7*L_neg/10));             % Identifies the banks whose pre-trade balances are the 70\% smallest (most negative) (finds the position of the banks with smallest balances)
    I_best_10percent=pos(end:-1:end-round(7*L_pos/10)+1);   % Identifies the banks whose pre-trade balances are the 70\% largest (most positive) (finds the position of the banks with largest balances)
    
    Times_worse=Times(I_worse_10percent,:);             % Meeting time corresponding to the banks whose pre-trade balances are the 70\% smallest (most negative)
    Times_best=Times(I_best_10percent,:);               % Meeting time corresponding to the banks whose pre-trade balances are the 70\% largest (most positive)
    M_worse=M(I_worse_10percent,1:end-1);               % Balance of the banks whose pre-trade balances are the 70\% smallest (most negative)
    M_best=M(I_best_10percent,1:end-1);                 % Balance of the banks whose pre-trade balances are the 70\% largest (most positive)
    PURCHASES_worse=PURCHASES(I_worse_10percent,:);     % Sales by banks whose pre-trade balances are the 70\% smallest (most negative)
    PURCHASES_best=PURCHASES(I_best_10percent,:);       % Purchases by banks whose pre-trade balances are the 70\% largest (most postive)
    
    % The following code controls for the possibility that a bank with negative balance first purchases reserves and then sales within the same minute. This is a problem since it sold being long 
    % but might be confused with selling when short. It keeps the sales when balances are negative and the purchased when balances are positve.
    index_w=find(Times_worse(:)>T/Slice_number*(i-1) & Times_worse(:)<=T/Slice_number*i & M_worse(:)<k_0+k_bar); % Identifies of the banks whose pre-trade balances are the 70\% smallest at the begining of the minute, the times (seconds) when the balance is still negative
    index_b=find(Times_best(:)>T/Slice_number*(i-1) & Times_best(:)<=T/Slice_number*i & M_best(:)>=k_0+k_bar);   % Identifies of the banks whose pre-trade balances are the 70\% largest at the begining of the minute, the times (seconds) when the balance is still positive
    SAL_w(1:length(index_w),i)=max(-PURCHASES_worse(index_w),0);    % Keeps the sales (amount) by banks whose pre-trade balances at the begining of the minute were the 70\% smallest (most negative) and are still negative
    PUR_b(1:length(index_b),i)=max(PURCHASES_best(index_b),0);      % Keeps the purchases (amount) by banks whose pre-trade balances at the begining of the minute were the 70\% largest (most postive) and are still postive
    L_SAL_w(i)=length(index_w);
    L_PUR_b(i)=length(index_b);
end
S_TRD=S_PUR+S_SAL;          % Total amount of transactions (sold and purchased) each minute (interval)
N_TRD=N_PUR+N_SAL;          % Total number of transactions (sold and purchased) each minute
AS_TRD=AS_PUR+AS_SAL;       % Cumulative amount of fed funds traded (sold and purchased)
AN_TRD=AN_PUR+AN_SAL;       % Cumulative number of fed funds traded (sold and purchased)
H_m_TRD=H_m_PUR+H_m_SAL;    % Number of transactions of sizes 1,2,3,...,floor(K/2) each minute

RHO_market_day=sum(sum(rho.*abs(PURCHASES)))/sum(sum(abs(PURCHASES))); % Market rate for the day (value-weighted)
RHO_market_day_FF=sum(sum(rho_FF.*abs(PURCHASES)))/sum(sum(abs(PURCHASES))); % Market rate for the day (value-weighted)

%% 'Average' measures of lending (sales), borrowing (purchases) and intermediation
mean_Purchases=zeros(1,K+1);
mean_Sales=zeros(1,K+1);
mean_X=zeros(1,K+1);
mean_Lenders=zeros(1,K+1);
mean_Borrowers=zeros(1,K+1);
mean_Counterparties=zeros(1,K+1);
mean_everCounterparties=zeros(1,K+1);

m_Purchases=zeros(1,K+1);
m_Sales=zeros(1,K+1);
m_X=zeros(1,K+1);
m_Lenders=zeros(1,K+1);
m_Borrowers=zeros(1,K+1);
m_Counterparties=zeros(1,K+1);
m_everCounterparties=zeros(1,K+1);

% Different measures per type of bank

% Calculates 'average' measures per type of bank
for i=0:K
    I=find(M(:,1)==i); % The positions of banks that started with with balance i (k_0 in the paper notation).
    mean_Purchases(i+1)=sum(Purchases(I))/length(I);    % Average purchases for banks with balances k_0, where k_0={0,1,2,3,4}
    mean_Sales(i+1)=sum(Sales(I))/length(I);            % Average sales for banks with balances k_0, where k_0={0,1,2,3,4}          
    mean_X(i+1)=sum(X(I))/length(I);                    % Average gross reallocation of funds for banks with balances k_0, where k_0={0,1,2,3,4}
    
    mean_Lenders(i+1)=sum(Lenders(I))/length(I);                % Average number of lenders banks with balances k_0 borrowed from, where k_0={0,1,2,3,4}   
    mean_Borrowers(i+1)=sum(Borrowers(I))/length(I);            % Average number of borrowers banks with balances k_0 lent to, where k_0={0,1,2,3,4}
    mean_Counterparties(i+1)=sum(Counterparties(I))/length(I);  % Average number of banks with balances k_0 that borrow and lend during a trading session, where k_0={0,1,2,3,4}

    mean_everCounterparties(i+1)=sum(everCounterparties(I))/length(I);  % Average number of banks with balances k_0 that borrow and lend during a trading session, where k_0={0,1,2,3,4} 
end

% Calculates aggregate measures per type of bank (e.g. total purchases by banks that started the trading session with k_0 balances, for k_0={0,1,2,3,4})
for i=0:K
    I=find(M(:,1)==i); % The positions of banks that started with with balance i (k_0 in the paper notation).
    m_Purchases(i+1)=sum(Purchases(I));    % Total purchases for banks with balances k_0, where k_0={0,1,2,3,4}
    m_Sales(i+1)=sum(Sales(I));            % Total sales for banks with balances k_0, where k_0={0,1,2,3,4}          
    m_X(i+1)=sum(X(I));                    % Total gross reallocation of funds for banks with balances k_0, where k_0={0,1,2,3,4}
    
    m_Lenders(i+1)=sum(Lenders(I));                 % Total number of lenders banks with balances k_0 borrowed from, where k_0={0,1,2,3,4}   
    m_Borrowers(i+1)=sum(Borrowers(I));             % Total number of borrowers banks with balances k_0 lent to, where k_0={0,1,2,3,4}   
    m_Counterparties(i+1)=sum(Counterparties(I));   % Total number of counterparties banks with balances k_0 have, where k_0={0,1,2,3,4}   

    m_everCounterparties(i+1)=sum(everCounterparties(I));  % Total number of banks with balances k_0 that borrow and lend during a trading session, where k_0={0,1,2,3,4}
end

%% Market-wide aggreate measures
% Aggregate measures
market_Purchases=m_Purchases*ones(1,length(m_ini))';         % Total purchases by all banks (market-wide)
market_Sales=m_Sales*ones(1,length(m_ini))';                 % Total sales by all banks (market-wide)
market_X=m_X*ones(1,length(m_ini))';                         % Total gross reallocation of funds by all banks (market-wide)
market_i=market_X/(market_Purchases+market_Sales);           % Proportion of intermediated funds (market-wide intermediation)

market_Lenders=m_Lenders*ones(1,length(m_ini))';                % Total number of lenders (market-wide)
market_Borrowers=m_Borrowers*ones(1,length(m_ini))';            % Total number of borrowers (market-wide)
market_Counterparties=m_Counterparties*ones(1,length(m_ini))';  % Total number of counterparties (market-wide)

market_everCounterparties=m_everCounterparties*ones(1,length(m_ini))';  % Total number of banks that borrow and lend (at least once) during a trading session

% Average measures (for the whole market)
mean_market_Purchases=market_Purchases/H;         % Purchases (amount) by the average bank
mean_market_Sales=market_Sales/H;                 % Sales (amount) by the average bank
mean_market_X=market_X/H;                         % Total gross reallocation of funds by the average bank
mean_market_i=market_i/H;                         % Proportion of intermediated funds by the average bank

mean_market_Lenders=market_Lenders/H;                % Average number of lenders (market-wide)
mean_market_Borrowers=market_Borrowers/H;            % Average number of borrowers (market-wide)
mean_market_Counterparties=market_Counterparties/H;  % Average number of counterparties (market-wide)

mean_market_everCounterparties=market_everCounterparties/H;  % Average number of banks that borrow and lend (at least once) during a trading session

for i=1:(K+1)
    auxil(i)=i-1-k_0;
end
assets_0=auxil*m_ini';

save(['simulation' num2str(cputime*100) '.mat'])
