% TREATMENT OF OUTLIERS 2007
% Implements Grubbs teast and a modified version of the Grubbs method


load data_2007_goodbanks.mat %%% Master file that contains 2007 master data: data_2007

% Histograms of original data (potentially with outliers)
HH=figure(1);
hist(mean_data_2007(:,1),50)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
ylabel('Number of banks', 'fontsize', 20)
xlabel('4:00 pm imputed balances/RoB', 'fontsize', 20)
title('2007')
print(['histogram_opening_balances_outliers_2007_4pm'],'-depsc2')


sided = 1;           % Two-sided distribution (default)
alpha = 0.0005;      % Confidence level (99.9%)
dist = 0;            % Data are assumed to be normal
nmin = 10;           % Minimum number of observations before the algorithm stops.


%% Tests if data are from a Gaussian distribution
parameters_mle=mle(mean_data_2007(:,1),'distribution','norm');
MUHAT_fullsample=parameters_mle(1);
SIGMAHAT_fullsample=parameters_mle(2);
[H,pval]=kstest(mean_data_2007(:,1),[mean_data_2007(:,1) normcdf(mean_data_2007(:,1),MUHAT_fullsample,SIGMAHAT_fullsample)],alpha)

%% Tests if data are from a mixture of 2 Gaussians
GM_mix2=gmdistribution.fit(mean_data_2007(:,1),2)
[H,pval]=kstest(mean_data_2007(:,1),[mean_data_2007(:,1) cdf(GM_mix2,mean_data_2007(:,1))],0.1) 

%% TREATMENT OF OUTLIERS
% Identifying outliers
%% Grubbs method
load xx_date_2007 
[Gmax_loop Gmin_loop G2_loop lambda1_loop lambda2_loop outlier outlier_num] = grubbs(mean_data_2007(:,1), xx_date_2007, sided, alpha, dist, nmin)
% Removing outliers identified by Grubbs' method
sort(outlier_num(:,1))
no_grubbs_outliers=mean_data_2007([1:9 11:12 14:28 31:108 110:127 129:end],1);

HH=figure(4);
boxplot(no_grubbs_outliers,'symbol','x')        % Plots the boxplot. x denotes outliners
x_mean = mean(no_grubbs_outliers);              % Calculates the mean
hold on
plot(x_mean(:),'go','MarkerFaceColor','g')      % Plots the mean. MarkerFaceColor fills color for closed-shape markers
H1=get(HH,'Children');
set(H1,'fontsize', 12);
title('2007')
tickLabelStr = {'4:00pm Balances/RoB'};            % Plots 4pm Balances/RoB under the box
text_h = findobj(gca, 'Type', 'text');    
    for cnt = 1:length(text_h)
        set(text_h(cnt),'FontSize',12,'String',tickLabelStr{length(tickLabelStr)-cnt+1},'HorizontalAlignment', 'center')
    end
hold off
print(['boxplot_ratio_nooutliers_2007'],'-depsc2')

%% Modified Grubbs' method
% Note: make sure the first component has the largest variance!
disp('Verify that the first component has the largest variance!')
GM_mix2.Sigma
removing_G2_2007 % Identifies outliers with respect to the Guassian with the largest variance


%% Box plots
HH=figure(3);
boxplot(mean_data_2007(:,1),'symbol','x')                       % Plots the boxplot. x denotes outliners
x_mean = mean(mean_data_2007(:,1));                             % Calculates the mean
hold on
plot(x_mean(:),'go','MarkerFaceColor','g')                      % Plots the mean. MarkerFaceColor fills color for closed-shape markers
H1=get(HH,'Children');
set(H1,'fontsize', 12);
title('2007')
tickLabelStr = {'4:00pm Balances/RoB'};                         % Plots 4pm Balances/RoB under the box
text_h = findobj(gca, 'Type', 'text');    
    for cnt = 1:length(text_h)
        set(text_h(cnt),'FontSize',12,'String',tickLabelStr{length(tickLabelStr)-cnt+1},'HorizontalAlignment', 'center')
    end
hold off
print(['boxplot_ratio_outliers_2007'],'-depsc2')





