% TREATMENT OF OUTLIERS 2011

load data_2011_goodbanks.mat %%% Master file that contains master data: data_2011

% Histograms of original data (potentially with outliers)
HH=figure(1);
hist(mean_data_2011(:,1),50)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
ylabel('Number of banks', 'fontsize', 20)
xlabel('4:00 pm imputed balances/RoB', 'fontsize', 20)
title('2011')
print(['histogram_opening_balances_outliers_2011_4pm'],'-depsc2')

sided = 1;           % Two-sided distribution (default)
alpha = 0.0005;      % Confidence level (99.9%)
dist = 0;            % Data are assumed to be normal
nmin = 10;           % Minimum number of observations before the algorithm stops.


%% Test if data are from a Gaussian distribution
% Computing parameter estimates 
parameters_mle=mle(mean_data_2011(:,1),'distribution','norm');
MUHAT_fullsample=parameters_mle(1);
SIGMAHAT_fullsample=parameters_mle(2);

% Kolmogorov-Smirnov
[H,pval]=kstest(mean_data_2011(:,1),[mean_data_2011(:,1) normcdf(mean_data_2011(:,1),MUHAT_fullsample,SIGMAHAT_fullsample)],0.1)

%% Test if data are from a mixture of 2 Gaussians
% Computing parameter estimates 
GM_mix2_2011=gmdistribution.fit(mean_data_2011(:,1),2)

% Kolmogorov-Smirnov
[H,pval]=kstest(mean_data_2011(:,1),[mean_data_2011(:,1) cdf(GM_mix2_2011,mean_data_2011(:,1))],0.1) 

%% Test if data are from a mixture of 3 Gaussians
% Computing parameter estimates 
GM_mix3=gmdistribution.fit(mean_data_2011(:,1),3)

% Kolmogorov-Smirnov
[H,pval]=kstest(mean_data_2011(:,1),[mean_data_2011(:,1) cdf(GM_mix3,mean_data_2011(:,1))],0.001) 


%% Test if data (minus the three largest values) are from a mixture of 2 Gaussians
% Computing parameter estimates 
GM_mix2_2011_nooutliers=gmdistribution.fit(mean_data_2011_nooutliers,2)

% Kolmogorov-Smirnov
[H,pval]=kstest(mean_data_2011_nooutliers,[mean_data_2011_nooutliers cdf(GM_mix2_2011_nooutliers,mean_data_2011_nooutliers)],0.001) 

%% TREATMENT OF OUTLIERS
% Identifying outliers
%% Grubbs method -- 2011 
load xx_date_2011 
[Gmax_loop Gmin_loop G2_loop lambda1_loop lambda2_loop outlier outlier_num] = grubbs(mean_data_2011(:,1), xx_date_2007, sided, alpha, dist, nmin)

% Removing outliers identified by Grubbs' method
sort(outlier_num(:,1))
no_grubbs_outliers=mean_data_2011([1:12 14:62 64:120 122:end],1);

HH=figure(4);
boxplot(no_grubbs_outliers,'symbol','x')        % Plots the boxplot. x denotes outliners
x_mean = mean(no_grubbs_outliers);              % Calculates the mean
hold on
plot(x_mean(:),'go','MarkerFaceColor','g')      % Plots the mean. MarkerFaceColor fills color for closed-shape markers
H1=get(HH,'Children');
set(H1,'fontsize', 12);
title('2011')
tickLabelStr = {'4:00pm Balances/RoB'};            % Plots 4pm Balances/RoB under the box
text_h = findobj(gca, 'Type', 'text');    
    for cnt = 1:length(text_h)
        set(text_h(cnt),'FontSize',12,'String',tickLabelStr{length(tickLabelStr)-cnt+1},'HorizontalAlignment', 'center')
    end
hold off
print(['boxplot_ratio_nooutliers_2011'],'-depsc2')


% Histograms of data without Grubbs outliers
HH=figure(5);
hist(no_grubbs_outliers,50)
grid
H1=get(HH,'Children');
set(H1(1),'fontsize', 15);
ylabel('Number of banks', 'fontsize', 20)
xlabel('4:00 pm imputed balances/RoB', 'fontsize', 20)
title('2011')
print(['histogram_opening_balances_nooutliers_2011_4pm'],'-depsc2')

%% Modified Grubbs' method
% Note: make sure the first component has the largest variance!
disp('Verify that the first component has the largest variance!')
GM_mix2_2011.Sigma
removing_G2_2011 % Identifies outliers with respect to the Guassian with the largest variance

%% Box plots
HH=figure(3);
boxplot(mean_data_2011(:,1),'symbol','x')                         % Plots the boxplot. x denotes outliners
x_mean = mean(mean_data_2011(:,1));                               % Calculates the mean
hold on
plot(x_mean(:),'go','MarkerFaceColor','g')      % Plots the mean. MarkerFaceColor fills color for closed-shape markers
H1=get(HH,'Children');
set(H1,'fontsize', 12);
title('2011')
tickLabelStr = {'4:00pm Balances/RoB'};            % Plots 4pm Balances/RoB under the box
text_h = findobj(gca, 'Type', 'text');    
    for cnt = 1:length(text_h)
        set(text_h(cnt),'FontSize',12,'String',tickLabelStr{length(tickLabelStr)-cnt+1},'HorizontalAlignment', 'center')
    end
hold off
print(['boxplot_ratio_outliers_2011'],'-depsc2')





